#include <windows.h>
#include <windowsx.h>
#include "odbutton.h"

#define ID_BUTTON  (1)

TCHAR g_lpszClassName[] = TEXT("_OD_BUTTON_");
HWND g_hWnd = NULL;
HWND g_hButton = NULL;


///////////////////////////////////////////////////////////////////////////////


int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd) {

	WNDCLASSEX wcl;

	wcl.cbSize = sizeof(WNDCLASSEX);
	wcl.hInstance = hInstance;
	wcl.lpszClassName = g_lpszClassName;
	wcl.lpfnWndProc = WndProc;
	wcl.style = NULL;
	wcl.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wcl.hIconSm = NULL;
	wcl.hCursor = LoadCursor(NULL, IDC_ARROW);
	wcl.lpszMenuName = NULL;
	wcl.cbClsExtra = 0;
	wcl.cbWndExtra = 0;
	wcl.hbrBackground = (HBRUSH) GetStockObject(GRAY_BRUSH);

	if(!RegisterClassEx(&wcl)) {

		return FALSE;
	}

	g_hWnd = CreateWindowEx( 
		0,  	
		g_lpszClassName,
		TEXT("Owner-Draw Button"),
		WS_OVERLAPPEDWINDOW,
		CW_USEDEFAULT, 	CW_USEDEFAULT,
		300, 150,
		NULL, 
		NULL, 
		hInstance, 
		NULL);

	if(!g_hWnd) {
		
		return FALSE;
	}

	ShowWindow( g_hWnd, nShowCmd);
	UpdateWindow(g_hWnd);

	MSG Msg;
	
	while(GetMessage(&Msg, NULL, 0, 0)) {
		TranslateMessage(&Msg);
		DispatchMessage(&Msg);
	}

	return Msg.wParam;
	
}


///////////////////////////////////////////////////////////////////////////////


LRESULT CALLBACK WndProc( HWND hwnd, UINT Msg, WPARAM wParam, LPARAM lParam) {
	
	switch(Msg) {
	HANDLE_MSG ( hwnd, WM_COMMAND, OnCommand );
	HANDLE_MSG ( hwnd, WM_CREATE, OnCreate );
	HANDLE_MSG ( hwnd, WM_DESTROY, OnDestroy);
	HANDLE_MSG ( hwnd, WM_DRAWITEM, OnDrawItem );
	HANDLE_MSG ( hwnd, WM_SIZE, OnSize );
	default:
		return DefWindowProc(hwnd, Msg, wParam, lParam);
	}

	return FALSE;
}


///////////////////////////////////////////////////////////////////////////////


void OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify) {

	if ( ID_BUTTON == id ) {
		::MessageBox ( hwnd, TEXT("Hello"), TEXT("Owner-Draw button"), MB_OK );
	}

}


///////////////////////////////////////////////////////////////////////////////


BOOL OnCreate(HWND hwnd, LPCREATESTRUCT lpCreateStruct) {

	g_hButton = CreateWindowEx (
		0,
		TEXT("button"),
		TEXT(""),
		WS_CHILD | WS_VISIBLE | BS_OWNERDRAW,
		0, 0, 0, 0,
		hwnd, (HMENU) ID_BUTTON, NULL, NULL );

	ShowWindow ( g_hButton, SW_SHOW );

	return TRUE;
}


///////////////////////////////////////////////////////////////////////////////


void OnDestroy( HWND hwnd ) {

	PostQuitMessage (0);
	
}


///////////////////////////////////////////////////////////////////////////////


void OnDrawItem(HWND hwnd, const DRAWITEMSTRUCT * lpDrawItem) {

	if ( ID_BUTTON != lpDrawItem->CtlID ) {
		return;
	}

	HDC hDC = lpDrawItem->hDC;
	BOOL bIsPressed = (lpDrawItem->itemState & ODS_SELECTED);
	BOOL bIsFocused  = (lpDrawItem->itemState & ODS_FOCUS);
	BOOL bIsDisabled = (lpDrawItem->itemState & ODS_DISABLED);

	RECT rectItem = lpDrawItem->rcItem;

	SetBkMode(hDC, TRANSPARENT);

	if (bIsFocused) {
		HBRUSH hBlackBrush = (HBRUSH) GetStockObject (BLACK_BRUSH);
		FrameRect (hDC, &rectItem, hBlackBrush);
		InflateRect (&rectItem, -1, -1);
	}

	// Button Face
	COLORREF clrButtonFace = GetSysColor (COLOR_BTNFACE);
	HBRUSH	hButtonFaceBrush = CreateSolidBrush( clrButtonFace );
	FillRect (hDC, &rectItem, hButtonFaceBrush);
	DeleteObject (hButtonFaceBrush);

	// Pressed Button
	if (bIsPressed) {
		HBRUSH brBtnShadow = CreateSolidBrush(GetSysColor(COLOR_BTNSHADOW));
		FrameRect (hDC, &rectItem, brBtnShadow);
		DeleteObject (brBtnShadow);
	}
	else {
		DrawFrameControl (hDC, &rectItem, DFC_BUTTON, DFCS_BUTTONPUSH);
	}

	// Focus Rect
	if ( bIsFocused ) {
		RECT rectFocus = rectItem;
	
		InflateRect (&rectFocus, -3, -3);
		if ( bIsPressed ) {
			OffsetRect ( &rectFocus, 1, 1);
		}
		DrawFocusRect (hDC, &rectFocus);
	}

	// Icon
	INT cxIcon = 10 + ( bIsPressed ? 1 : 0 );
	INT cyIcon = 8 + ( bIsPressed ? 1 : 0);
	HICON hIcon = LoadIcon ( NULL, IDI_QUESTION);
	::DrawIcon (hDC, cxIcon, cyIcon, hIcon);

}


///////////////////////////////////////////////////////////////////////////////


void OnSize(HWND hwnd, UINT state, int cx, int cy) {

	MoveWindow ( g_hButton, 10, 10, 100, 50, TRUE);

}
